/***************************************************************************
                          stuff.cpp  -  description
                             -------------------
    begin                : Sun May 6 2001
    copyright            : (C) 2001 by Stefan Winter
    email                : mail@stefan-winter.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "config.h"
#include "interface_wireless_wirelessextensions.h"
#include <iwlib.h>
#include <qdir.h>
#include <qfile.h>
#include <qstringlist.h>
#include <arts/artsflow.h>
#include <arts/connect.h>
#include <arts/iomanager.h>
#include <arts/referenceclean.h>
#include <iostream>
#include <string>
#include <klocale.h>
#include <kprocio.h>
#include <kdebug.h>
#include <qstring.h>
#include <arpa/inet.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <unistd.h>


// domi:the wireless-tools people managed to change their interfaces
// around.  Someone with more knowledge of this code should look into
// moving to use the new iw_range interfaces.
#if HAVE_IW_27
inline int
wtc_private_iw_get_stats (int skfd, char *ifname, iwstats * stats)
{
  return iw_get_stats (skfd, ifname, stats, 0, 0);
}
#else
inline int
wtc_private_iw_get_stats (int skfd, char *ifname, iwstats * stats)
{
  return iw_get_stats (skfd, ifname, stats);
}
#endif

/* ================================== FROM IWCONFIG.C ================================== */
int
Interface_wireless_wirelessextensions::get_info (int skfd, const char *interface,
						 struct wireless_info *info)
{
  struct iwreq wrq;
  char * ifname = (char *) interface;
  memset ((char *) info, 0, sizeof (struct wireless_info));

  /* Get wireless name */
  if (iw_get_ext (skfd, ifname, SIOCGIWNAME, &wrq) < 0)
    {
      /* If no wireless name : no wireless extensions */
      /* But let's check if the interface exists at all */
      struct ifreq ifr;

      strcpy (ifr.ifr_name, ifname);
      if (ioctl (skfd, SIOCGIFFLAGS, &ifr) < 0)
	return (-ENODEV);
      else
	return (-ENOTSUP);
    }
  else
    {
      strncpy (info->name, wrq.u.name, IFNAMSIZ);
      info->name[IFNAMSIZ] = '\0';
    }

  /* Get ranges */
  if (iw_get_range_info (skfd, ifname, &(info->range)) >= 0)
    info->has_range = 1;

  /* Get network ID */
  if (iw_get_ext (skfd, ifname, SIOCGIWNWID, &wrq) >= 0)
    {
      info->has_nwid = 1;
      memcpy (&(info->nwid), &(wrq.u.nwid), sizeof (iwparam));
    }

  /* Get frequency / channel */
  if (iw_get_ext (skfd, ifname, SIOCGIWFREQ, &wrq) >= 0)
    {
      info->has_freq = 1;
      info->freq = iw_freq2float (&(wrq.u.freq));
    }

  /* Get sensitivity */
  if (iw_get_ext (skfd, ifname, SIOCGIWSENS, &wrq) >= 0)
    {
      info->has_sens = 1;
      memcpy (&(info->sens), &(wrq.u.sens), sizeof (iwparam));
    }

  /* Get encryption information */
  wrq.u.data.pointer = (caddr_t) info->key;
  wrq.u.data.length = IW_ENCODING_TOKEN_MAX;
  wrq.u.data.flags = 0;
  if (iw_get_ext (skfd, ifname, SIOCGIWENCODE, &wrq) >= 0)
    {
      info->has_key = 1;
      info->key_size = wrq.u.data.length;
      info->key_flags = wrq.u.data.flags;
    }

  /* Get ESSID */
  wrq.u.essid.pointer = (caddr_t) info->essid;
  wrq.u.essid.length = IW_ESSID_MAX_SIZE + 1;
  wrq.u.essid.flags = 0;
  if (iw_get_ext (skfd, ifname, SIOCGIWESSID, &wrq) >= 0)
    {
      info->has_essid = 1;
      info->essid_on = wrq.u.data.flags;
    }

  /* Get AP address */
  if (iw_get_ext (skfd, ifname, SIOCGIWAP, &wrq) >= 0)
    {
      info->has_ap_addr = 1;
      memcpy (&(info->ap_addr), &(wrq.u.ap_addr), sizeof (sockaddr));
    }

  /* Get NickName */
  wrq.u.essid.pointer = (caddr_t) info->nickname;
  wrq.u.essid.length = IW_ESSID_MAX_SIZE + 1;
  wrq.u.essid.flags = 0;
  if (iw_get_ext (skfd, ifname, SIOCGIWNICKN, &wrq) >= 0)
    if (wrq.u.data.length > 1)
      info->has_nickname = 1;

  /* Get bit rate */
  if (iw_get_ext (skfd, ifname, SIOCGIWRATE, &wrq) >= 0)
    {
      info->has_bitrate = 1;
      memcpy (&(info->bitrate), &(wrq.u.bitrate), sizeof (iwparam));
    }

  /* Get RTS threshold */
  if (iw_get_ext (skfd, ifname, SIOCGIWRTS, &wrq) >= 0)
    {
      info->has_rts = 1;
      memcpy (&(info->rts), &(wrq.u.rts), sizeof (iwparam));
    }

  /* Get fragmentation threshold */
  if (iw_get_ext (skfd, ifname, SIOCGIWFRAG, &wrq) >= 0)
    {
      info->has_frag = 1;
      memcpy (&(info->frag), &(wrq.u.frag), sizeof (iwparam));
    }

  /* Get operation mode */
  if (iw_get_ext (skfd, ifname, SIOCGIWMODE, &wrq) >= 0)
    {
      info->mode = wrq.u.mode;
      if ((info->mode < IW_NUM_OPER_MODE) && (info->mode >= 0))
	info->has_mode = 1;
    }

  /* Get Power Management settings */
  wrq.u.power.flags = 0;
  if (iw_get_ext (skfd, ifname, SIOCGIWPOWER, &wrq) >= 0)
    {
      info->has_power = 1;
      memcpy (&(info->power), &(wrq.u.power), sizeof (iwparam));
    }

#if WIRELESS_EXT > 9
  /* Get Transmit Power */
  if (iw_get_ext (skfd, ifname, SIOCGIWTXPOW, &wrq) >= 0)
    {
      info->has_txpower = 1;
      memcpy (&(info->txpower), &(wrq.u.txpower), sizeof (iwparam));
    }
#endif

#if WIRELESS_EXT > 10
  /* Get retry limit/lifetime */
  if (iw_get_ext (skfd, ifname, SIOCGIWRETRY, &wrq) >= 0)
    {
      info->has_retry = 1;
      memcpy (&(info->retry), &(wrq.u.retry), sizeof (iwparam));
    }
#endif /* WIRELESS_EXT > 10 */

  /* Get stats */
  if (wtc_private_iw_get_stats (skfd, (char *) ifname, &(info->stats)) >= 0)
    {
      info->has_stats = 1;
    }

  return (0);
}

/* ================================ END IWCONFIG.C ================================ */


Interface_wireless_wirelessextensions::Interface_wireless_wirelessextensions (QStringList * ignoreInterfaces):Interface_wireless
  (ignoreInterfaces)
{
};

bool Interface_wireless_wirelessextensions::poll_device_info ()
{
  wireless_info *
    tempic1 = new (wireless_info);
  iwstats *
    tempic2 = new (iw_statistics);
  int socket = iw_sockets_open ();
  if (socket < 0)
    return false;
  (current < MAX_HISTORY-1) ? current++ : current = 0;
  if (interface_name == "")
    {
      // this instance does not yet have an interface to care about ...
      QString ignoreList = ignoreInterfaces->join(" ");
      if (!already_warned)
	kdDebug () << "Autodetecting...";
      QFile
      procnetdev ("/proc/net/dev");
      procnetdev.open (IO_ReadOnly);
      QStringList
	liste;
      QString
	temp = "";
      QString
	temptrash = "X";
      int
	end = 1;
      while ((end != 0) && (temp != temptrash))
	{
	  end = procnetdev.readLine (temp, 9999);
	  temptrash = temp;
	  temp.truncate (temp.find (":") > 0 ? temp.find (":") : 0);
	  temp = temp.stripWhiteSpace ();
	  if ((end != 0) && (temp != ""))
	    liste += temp;
	}
      for (QStringList::Iterator it = liste.begin (); it != liste.end ();
	   ++it)
	{
	  if (!already_warned)
	    kdDebug () << "[ " << (*it).latin1 () << " ] ";
	  char
	    device[128];
	  snprintf (device, 128, "%s", (*it).latin1 ());
	  int
	    result = get_info (socket, device, tempic1);
	  if ((result != -ENODEV) && (result != -ENOTSUP) && (ignoreList.find(device)==-1))
	    {
	      char
		device[128];
	      snprintf (device, 128, "%s", (*it).latin1 ());
	      kdDebug () << "Found!\n";
	      interface_name = device;
	      emit
	      strengthChanged ();
	      emit
	      statusChanged ();
	      emit
	      modeChanged ();
	      emit
	      speedChanged ();
	      valid[current] = false;
	      emit statsUpdated ();
	    }
	}
      if (interface_name == "")
	{
	  kdDebug () << "No wireless interface found.\n";
	  already_warned = true;
	  close(socket);
	  return false;
	}
    }
  else
    {				// an interface name is already assigned
      int
	result = get_info (socket, interface_name.latin1 (), tempic1);
      if ((result == -ENODEV) || (result == -ENOTSUP))
	{
	  // interface has disappeared - unplugged?
	  // reset all info except stats
	    has_frequency = false;
  interface_name = "";
  frequency = 0.;
  has_mode = false;
  mode = 0;
  has_key = 0;
  key = "";
  key_size = 0;
  key_flags = 0;
  essid = "";
  access_point_address = "";
  ip_address = "";
  bitrate = 0.;
  socket = -1;
  has_range = false;
	  // propagate the changes
	  already_warned = true;
	  emit
	  strengthChanged ();
	  emit
	  statusChanged ();
	  emit
	  modeChanged ();
	  emit
	  speedChanged ();
	  valid[current] = false;
	  emit
	  statsUpdated ();
	  close(socket);
	  return false;
	}
    }
  bool
    emitStatusChanged = false, emitModeChanged = false, emitEssidChanged =
    false, emitSpeedChanged = false, emitStrengthChanged = false;
    wtc_private_iw_get_stats (socket,(char *) interface_name.latin1 (), tempic2);
  already_warned = false;
  has_frequency = tempic1->has_freq;
  if (has_frequency)
    {
      if (frequency != tempic1->freq)
	emitStatusChanged = true;
      frequency = tempic1->freq;
    }
  has_mode = tempic1->has_mode;
  if (has_mode)
    {
      if (mode != tempic1->mode)
	emitModeChanged = true;
      mode = tempic1->mode;
    }
  has_key = tempic1->has_key;
  if (has_key)
    {
      if ((key != (char *) tempic1->key) || (key_size != tempic1->key_size)
	  || (key_flags != tempic1->key_flags))
	emitStatusChanged = true;
      key = (char *) tempic1->key;
      key_size = tempic1->key_size;
      key_flags = tempic1->key_flags;
    }
  if (essid != tempic1->essid)
    {
      emitStatusChanged = true;
      emitEssidChanged = true;
    }
  essid = (tempic1->essid_on ? tempic1->essid : "any");
  char
    ap_addr[256];
  iw_pr_ether (ap_addr, (const unsigned char *) tempic1->ap_addr.sa_data);
  if (access_point_address != ap_addr)
    emitStatusChanged = true;
  access_point_address = ap_addr;
  if (bitrate != tempic1->bitrate.value)
    emitSpeedChanged = true;
  bitrate = tempic1->bitrate.value;
  has_range = tempic1->has_range;
  if (has_range)
    range = tempic1->range.max_qual.qual;
  sigLevel[current] = tempic2->qual.level;
  if (tempic2->qual.level < sigLevelMin)
    sigLevelMin = tempic2->qual.level;
  if (tempic2->qual.level > sigLevelMax)
    sigLevelMax = tempic2->qual.level;
  noiseLevel[current] = tempic2->qual.noise;
  if (tempic2->qual.noise < noiseLevelMin)
    noiseLevelMin = tempic2->qual.noise;
  if (tempic2->qual.noise > noiseLevelMax)
    noiseLevelMax = tempic2->qual.noise;
  int
    tempqual = tempic2->qual.qual;
  if (has_range && (range != 0))
    tempqual = tempqual * 100 / range;
  if (      qual[current != 0 ? current - 1 : MAX_HISTORY-1] != tempqual)
    emitStrengthChanged = true;
  if (  sigLevel[current != 0 ? current - 1 : MAX_HISTORY-1] != tempic2->qual.level)
    emitStrengthChanged = true;
  if (noiseLevel[current != 0 ? current - 1 : MAX_HISTORY-1] != tempic2->qual.noise)
    emitStrengthChanged = true;
  qual[current] = tempqual;
  valid[current] = true; 

  // try to get our local IP address
  struct sockaddr *
    sa;
  struct sockaddr_in *
    sin;
  struct ifreq
    ifr;
  /* Copy the interface name into the buffer */
  strncpy (ifr.ifr_name, interface_name.latin1 (), IFNAMSIZ);

  if (ioctl (socket, SIOCGIFADDR, &ifr) == -1)
    {
      if (ip_address != "unavailable")
	emitStatusChanged = true;
      ip_address = "unavailable";
    }
  /* Now the buffer will contain the information we requested */
  sa = (struct sockaddr *) &(ifr.ifr_addr);
  if (sa->sa_family == AF_INET)
    {
      sin = (struct sockaddr_in *) sa;
      if (ip_address != (QString) inet_ntoa (sin->sin_addr))
	emitStatusChanged = true;
      ip_address = (QString) inet_ntoa (sin->sin_addr);
    }
  else
    {
      ip_address = "unavailable";
    }
  delete
    tempic1;
  delete
    tempic2;
  if (emitStatusChanged)
    emit
    statusChanged ();
  if (emitStrengthChanged)
    emit
    strengthChanged ();
  if (emitModeChanged)
    emit
    modeChanged ();
  if (emitSpeedChanged)
    emit
    speedChanged ();
  if (emitEssidChanged)
    emit
    essidChanged (essid);
  emit statsUpdated ();
    close(socket);
  return true;
}

QString Interface_wireless_wirelessextensions::print_scanning_token (struct iw_event * event)
{
  QString
    result;
  /* Now, let's decode the event */
  switch (event->cmd)
    {
    case SIOCGIWESSID:
      {
	char
	  essid[IW_ESSID_MAX_SIZE + 1];
	if ((event->u.essid.pointer) && (event->u.essid.length))
	  memcpy (essid, event->u.essid.pointer, event->u.essid.length);
	essid[event->u.essid.length] = '\0';
	if (event->u.essid.flags) {
	  result = essid;
	  }
      }
      break;
    default:
      break;

    }				/* switch(event->cmd) */
  /* May have changed */
  return result;
}


QStringList Interface_wireless_wirelessextensions::get_available_networks ()
{
   QStringList
      networks;
  {
    struct iwreq
      wrq;
    unsigned char
      buffer[IW_SCAN_MAX_DATA];	/* Results */
    struct timeval
      tv;			/* Select timeout */
    int
      timeout = 5000000;	/* 5s */

    if (interface_name=="") return networks;
      
    /* Init timeout value -> 250ms */
    tv.tv_sec = 0;
    tv.tv_usec = 250000;

    /*
     * Here we should look at the command line args and set the IW_SCAN_ flags
     * properly
     */
    wrq.u.param.flags = IW_SCAN_DEFAULT;
    wrq.u.param.value = 0;	/* Later */

    /* Initiate Scanning */
    if (iw_set_ext
	(socket, (char *) interface_name.latin1 (), SIOCSIWSCAN, &wrq) < 0)
      {
	if (errno != EPERM)
	  {
	    fprintf (stderr, "Interface doesn't support scanning.\n");
	    return networks;
	  }
	/* If we don't have the permission to initiate the scan, we may
	 * still have permission to read left-over results.
	 * But, don't wait !!! */
	tv.tv_usec = 0;
      }
    timeout -= tv.tv_usec;

    /* Forever */
    while (1)
      {
	fd_set
	  rfds;			/* File descriptors for select */
	int
	  last_fd;		/* Last fd */
	int
	  ret;

	/* Guess what ? We must re-generate rfds each time */
	FD_ZERO (&rfds);
	last_fd = -1;

	/* In here, add the rtnetlink fd in the list */

	/* Wait until something happens */
	// ret = select (last_fd + 1, &rfds, NULL, NULL, &tv);
	ret = sleep(3);

	/* Check if there was an error */
	if (ret > 0)
	  {
	    if (errno == EAGAIN || errno == EINTR)
	      continue;
	    fprintf (stderr, "Unhandled signal - aborting scan...\n");
	    return networks;
	  }

	/* Check if there was a timeout */
	if (ret == 0)
	  {
	    /* Try to read the results */
	    wrq.u.data.pointer = (char *) buffer;
	    wrq.u.data.flags = 0;
	    wrq.u.data.length = sizeof (buffer);
	    if (iw_get_ext
		(socket, (char *) interface_name.latin1 (), SIOCGIWSCAN,
		 &wrq) < 0)
	      {
		/* Check if results not available yet */
		// if (errno == EAGAIN)
		  // {
		    /* Restart timer for only 100ms */
		    //tv.tv_sec = 0;
		    //tv.tv_usec = 100000;
		    //timeout -= tv.tv_usec;
		    //if (timeout > 0)
		    //  continue;	/* Try again later */
		  //}

		/* Bad error */
		fprintf (stderr,
			 "Failed to read scan data - aborting scan...\n");
		return networks;
	      }
	    else
	      /* We have the results, go to process them */
	      break;
	  }

	/* In here, check if event and event type
	 * if scan event, read results. All errors bad & no reset timeout */
      }

    if (wrq.u.data.length)
      {
	struct iw_event
	  iwe;
	struct stream_descr
	  stream;
	int
	  ret;

	iw_init_event_stream (&stream, (char *) buffer, wrq.u.data.length);
	do
	  {
	    /* Extract an event and print it */
	    ret = iw_extract_event_stream (&stream, &iwe);
	    if (ret > 0) {
	      QString tempnet = print_scanning_token (&iwe);
	      if (!tempnet.isEmpty()) networks += tempnet;
		}
	  }
	while (ret > 0);
	printf ("\n");
      }
    return networks;
  }

  return networks;
}
